#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"


#define VENDOR_RQ_GET_LIGHT0 0
#define VENDOR_RQ_GET_LIGHT1 1
#define VENDOR_RQ_SET_LED    2
#define VENDOR_RQ_GET_LIGHT3 3
#define VENDOR_RQ_GET_LIGHT4 4
#define VENDOR_RQ_GET_LIGHT5 5



/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
/* ------------------------------------------------------------------------- */

uint8_t in_c(uint8_t pin)
{
    // read logic level of port-C's pin
    DDRC &= ~(1<<pin);
    return (PINC >>pin) & 1;
} 

void out_b(uint8_t pin, uint8_t val)
{
    // write logic level to port-B's pin
    DDRB |= 1<<pin;
    if (val)
        PORTB |= 1<<pin;
    else
        PORTB &= ~(1<<pin);
}

uint16_t read_adc(uint8_t channel)
{
    ADMUX = 0b01000000; 
    ADMUX |= channel;   
    ADCSRA = 0b11000110;

    while ((ADCSRA & (1<<ADSC))) 
        ;
    return ADCL + ADCH*256;  
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t dataBuffer[2];  /* must stay valid when usbFunctionSetup returns */

    if (rq->bRequest == VENDOR_RQ_SET_LED)
    {
        uint8_t led_no  = rq->wValue.bytes[0];
        uint8_t led_val = rq->wValue.bytes[1];
        out_b(led_no, led_val);
        return 0;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_LIGHT0)
    {
		
		uint16_t light = read_adc(0);
		dataBuffer[0] = light;
		dataBuffer[1] =	light>>8;
		usbMsgPtr = dataBuffer;
		return 2;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_LIGHT1)
    {
		
		uint16_t light = read_adc(1);
		dataBuffer[0] = light;
		dataBuffer[1] =	light>>8;
		usbMsgPtr = dataBuffer;
		return 2;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_LIGHT3)
    {
		
		uint16_t light = read_adc(3);
		dataBuffer[0] = light;
		dataBuffer[1] =	light>>8;
		usbMsgPtr = dataBuffer;
		return 2;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_LIGHT4)
    {
		
		uint16_t light = read_adc(4);
		dataBuffer[0] = light;
		dataBuffer[1] =	light>>8;
		usbMsgPtr = dataBuffer;
		return 2;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_LIGHT5)
    {
		
		uint16_t light = read_adc(5);
		dataBuffer[0] = light;
		dataBuffer[1] =	light>>8;
		usbMsgPtr = dataBuffer;
		return 2;
    }
    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{

    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(300);
    usbDeviceConnect();
    sei();
    for(;;){                /* main event loop */
        usbPoll();
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
